@@ -1,6 +1,6 @@
 package PDF::API2::Basic::PDF::Filter::FlateDecode;
 
-our $VERSION = '2.026'; # VERSION
+our $VERSION = '2.027'; # VERSION
 
 use base 'PDF::API2::Basic::PDF::Filter';
 
@@ -68,6 +68,10 @@ sub _depredict_png {
     my ($self, $stream) = @_;
     my $param  = $self->{'DecodeParms'};
 
+    my $prev = '';
+    $stream = $self->{'_depredict_next'} . $stream if defined $self->{'_depredict_next'};
+    $prev   = $self->{'_depredict_prev'}           if defined $self->{'_depredict_prev'};
+
     my $alpha   = $param->{Alpha}            ? $param->{Alpha}->val()            : 0;
     my $bpc     = $param->{BitsPerComponent} ? $param->{BitsPerComponent}->val() : 8;
     my $colors  = $param->{Colors}           ? $param->{Colors}->val()           : 1;
@@ -78,58 +82,67 @@ sub _depredict_png {
     my $bpp      = ceil($bpc * $comp / 8);
     my $scanline = 1 + ceil($bpp * $columns);
 
-    my $prev='';
-    my $clearstream='';
-    my $lastrow=($height||(length($stream)/$scanline))-1;
-    foreach my $n (0..$lastrow) {
+    my $clearstream = '';
+    my $lastrow = ($height || int(length($stream) / $scanline)) - 1;
+    foreach my $n (0 .. $lastrow) {
         # print STDERR "line $n:";
-        my $line=substr($stream,$n*$scanline,$scanline);
-        my $filter=vec($line,0,8);
-        my $clear='';
-        $line=substr($line,1);
-        # print STDERR " filter=$filter";
-        if($filter==0) {
-            $clear=$line;
-        } elsif($filter==1) {
-            foreach my $x (0..length($line)-1) {
-                vec($clear,$x,8)=(vec($line,$x,8)+vec($clear,$x-$bpp,8))%256;
-            }
-        } elsif($filter==2) {
-            foreach my $x (0..length($line)-1) {
-                vec($clear,$x,8)=(vec($line,$x,8)+vec($prev,$x,8))%256;
-            }
-        } elsif($filter==3) {
-            foreach my $x (0..length($line)-1) {
-                vec($clear,$x,8)=(vec($line,$x,8)+floor((vec($clear,$x-$bpp,8)+vec($prev,$x,8))/2))%256;
-            }
-        } elsif($filter==4) {
-            # die "paeth/png filter not supported.";
-            foreach my $x (0..length($line)-1) {
-                vec($clear,$x,8)=(vec($line,$x,8)+_paeth_predictor(vec($clear,$x-$bpp,8),vec($prev,$x,8),vec($prev,$x-$bpp,8)))%256;
+        my $line = substr($stream, $n * $scanline, $scanline);
+        my $filter = vec($line, 0, 8);
+        my $clear = '';
+        $line = substr($line, 1);
+        # print STDERR " filter=$filter ";
+        if ($filter == 0) {
+            $clear = $line;
+        }
+        elsif ($filter == 1) {
+            foreach my $x (0 .. length($line) - 1) {
+                vec($clear, $x, 8) = (vec($line, $x, 8) + vec($clear, $x - $bpp, 8)) % 256;
             }
         }
-        $prev=$clear;
-        foreach my $x (0..($columns*$comp)-1) {
-            vec($clearstream,($n*$columns*$comp)+$x,$bpc)=vec($clear,$x,$bpc);
-        #    print STDERR "".vec($clear,$x,$bpc).",";
+        elsif ($filter == 2) {
+            foreach my $x (0 .. length($line) - 1) {
+                vec($clear, $x, 8) = (vec($line, $x, 8) + vec($prev, $x, 8)) % 256;
+            }
+        }
+        elsif ($filter == 3) {
+            foreach my $x (0 .. length($line) - 1) {
+                vec($clear, $x, 8) = (vec($line, $x, 8) + floor((vec($clear, $x - $bpp, 8) + vec($prev, $x, 8)) / 2)) % 256;
+            }
+        }
+        elsif ($filter == 4) {
+            foreach my $x (0 .. length($line) - 1) {
+                vec($clear, $x, 8) = (vec($line, $x, 8) + _paeth_predictor(vec($clear, $x - $bpp, 8), vec($prev, $x, 8), vec($prev, $x - $bpp, 8))) % 256;
+            }
+        }
+        else {
+            die "Unexpected depredictor algorithm $filter requested on line $n (valid options are 0-4)";
+        }
+        $prev = $clear;
+        foreach my $x (0 .. ($columns * $comp) - 1) {
+            vec($clearstream, ($n * $columns * $comp) + $x, $bpc) = vec($clear, $x, $bpc);
+            # print STDERR "" . vec($clear, $x, $bpc) . ",";
         }
         # print STDERR "\n";
     }
+    $self->{'_depredict_next'} = substr($stream, ($lastrow + 1) * $scanline);
+    $self->{'_depredict_prev'} = $prev;
 
     return $clearstream;
 }
 
 sub _paeth_predictor {
-    my ($a, $b, $c)=@_;
+    my ($a, $b, $c) = @_;
     my $p = $a + $b - $c;
     my $pa = abs($p - $a);
     my $pb = abs($p - $b);
     my $pc = abs($p - $c);
-    if(($pa <= $pb) && ($pa <= $pc)) {
+    if ($pa <= $pb && $pa <= $pc) {
         return $a;
-    } elsif($pb <= $pc) {
+    }
+    elsif ($pb <= $pc) {
         return $b;
-    } else {
+    }
+    else {
         return $c;
     }
 }
