@@ -1,8 +1,11 @@
 package PDF::API2::Resource::Font::CoreFont::georgiabolditalic;
 
-our $VERSION = '2.026'; # VERSION
+our $VERSION = '2.027'; # VERSION
 
-$FONTDATA = {
+use strict;
+use warnings;
+
+our $FONTDATA = {
     'fontname' => 'Georgia,BoldItalic',
     'type' => 'TrueType',
     'apiname' => 'GeBoIt',
