@@ -1,8 +1,11 @@
 package PDF::API2::Resource::Font::CoreFont::helveticaboldoblique;
 
-our $VERSION = '2.026'; # VERSION
+our $VERSION = '2.027'; # VERSION
 
-$FONTDATA = {
+use strict;
+use warnings;
+
+our $FONTDATA = {
     'fontname' => 'Helvetica-BoldOblique',
     'type' => 'Type1',
     'apiname' => 'HeBi',
