@@ -1,8 +1,11 @@
 package PDF::API2::Resource::Font::CoreFont::trebuchetitalic;
 
-our $VERSION = '2.026'; # VERSION
+our $VERSION = '2.027'; # VERSION
 
-$FONTDATA = {
+use strict;
+use warnings;
+
+our $FONTDATA = {
     'fontname' => 'TrebuchetMS,Italic',
     'type' => 'TrueType',
     'apiname' => 'TrIt',
