@@ -1,13 +1,14 @@
 package PDF::API2::Resource::XObject::Form::BarCode;
 
-our $VERSION = '2.026'; # VERSION
+our $VERSION = '2.027'; # VERSION
 
 use base 'PDF::API2::Resource::XObject::Form::Hybrid';
 
 use PDF::API2::Util;
 use PDF::API2::Basic::PDF::Utils;
 
-no warnings qw[ deprecated recursion uninitialized ];
+use strict;
+use warnings;
 
 =head1 NAME
 
